/*
 * EepromController.c
 *
 * Created: 30.12.2012 17:34:09
 *  Author: Michael
 */ 

#include <avr/eeprom.h>
#include "EepromController.h"


//*****************************************************************************/
// Defines
//*****************************************************************************/
#define EEPReadByte(addr)         eeprom_read_byte((uint8_t *)addr)
#define EEPWriteByte(addr, val)   eeprom_write_byte((uint8_t *)addr, val)

#define EEPROM_DEF 0xFF

//*****************************************************************************/
// Address defines
//*****************************************************************************/
#define EEPROM_PIN_ADDRESS        0x0010      // needed EEPROM_MAX_PIN_SIZE

//////////////////////////////////////////////////////////////////////////////// 
/// 
/// \brief          Internal function to read null-terminated strings from a given
///                 address
/// 
/// \fn             BOOL ReadStringFromEEPROM(char* p_szBuffer, uint8_t p_nBuffSize, BOOL p_bDeleteOnError) 
/// 
/// \author         Michael
/// \date           30.12.2012 
/// 
/// \remarks        n/a 
/// 
/// \param[in]      p_nAddr           The EEPROM start address of the string
/// \param[out]     p_szPIN           The buffer which has to hold the read value
/// \param[in]      p_nBuffSize       The - already allocated - size of the buffer
/// \param[in]      p_bDeleteOnError  The flag, if already read values should be deleted again in case of error
/// 
/// \return         BOOL              TRUE, if the string could be read; FALSE otherwise
/// 
/// \version        1.0 
/// 
//////////////////////////////////////////////////////////////////////////////// 
static BOOL ReadStringFromEEPROM(uint16_t p_nAddr, char* p_szBuffer, uint8_t p_nBuffSize, BOOL p_bDeleteOnError)
{
  BOOL bRetVal = FALSE;
  uint8_t nCurrChar = 0x00;
  uint8_t nLength   = 0;
  char* pBufferStartAddress = p_szBuffer;
   
  char buffer[50];
  char test;
   
  do 
  {
    // 1. Read next byte and increment next address and the length value
    nCurrChar = EEPReadByte(p_nAddr++);
    nLength++;
    
    // 2. Add current character to output array and increment output pointer
    *p_szBuffer++ = nCurrChar;

  // 3. Read until \0 or a buffer overflow would occur.
  } while ( (nCurrChar != '\0') &&
            (nLength <= p_nBuffSize) );
  
  // 4. Reading from EEPROM finished; check result. If nChurrChar is '\0', then
  // the complete string could be read; if it's not '\0', the other abort condition
  // (nIdx <= p_nBuffSize) has occurred and that means, that the string could not
  // be read successfully. So delete already read values.
  if ('\0' == nCurrChar)
  {
    // Everything is fine. String value was read completely
    bRetVal = TRUE;
  }
  else if (TRUE == p_bDeleteOnError)
  {
    memset(pBufferStartAddress, EEPROM_DEF, p_nBuffSize);
    bRetVal = FALSE;
  }
    
  return bRetVal;
}

//////////////////////////////////////////////////////////////////////////////// 
/// 
/// \brief          Internal function to write a null-terminated strings to the 
///                 given address
/// 
/// \fn             void WriteStringToEEPROM(uint16_t p_nAddr, char* p_szBuffer, BOOL p_bUseUpdateMethod) 
/// 
/// \author         Michael
/// \date           30.12.2012 
/// 
/// \remarks        n/a 
/// 
/// \param[in]      p_nAddr             The address, where the string has to be stored
/// \param[in]      p_szBuffer          The null terminated string
/// \param[in]      p_bUseUpdateMethod  If TRUE, each byte will be read before overwritten and
///                                     it has already the "new" value, no write process will be done
/// 
/// \return         void 
/// 
/// \version        1.0 
/// 
//////////////////////////////////////////////////////////////////////////////// 
static void WriteStringToEEPROM(uint16_t p_nAddr, char* p_szBuffer, BOOL p_bUseUpdateMethod)
{
  uint8_t nCurrDestChar = 0x00;
  uint8_t nCurrSrcChar  = 0x00;
  BOOL bWriteByte = TRUE;
  uint8_t nLength   = 0;
  
  // Write up to - and including - the \0 termination.
  do 
  {
    // 1. Reset UpdateMode flag
    bWriteByte = TRUE;

    // 2. Get next character from given string
    nCurrSrcChar = *p_szBuffer;
    
    if (TRUE == p_bUseUpdateMethod)
    {
      // This method is for reducing the EEPROM write processes; but it needs more time
      nCurrDestChar = EEPReadByte(p_nAddr);
      
      if (nCurrSrcChar == nCurrDestChar)
      {
        // This address already contains the correct value; writing is not necessary)
        bWriteByte = FALSE;
      }
    }

    // 3. Write (or write not in update mode if value already correct) the current source character
    if (TRUE == bWriteByte)
    {
      EEPWriteByte(p_nAddr, nCurrSrcChar);
    }
    
    // 4. Increment address and source pointer
    p_nAddr++;
    p_szBuffer++;
  } while (nCurrSrcChar != '\0');
}

//////////////////////////////////////////////////////////////////////////////// 
/// 
/// \brief          This function read the stored PIN from the EEPROM and returns
///                 its value as null terminated string
/// 
/// \fn             BOOL E2PROM_ReadPinValue(char* p_szPIN, uint8_t p_nBuffSize) 
/// 
/// \author         Michael
/// \date           30.12.2012 
/// 
/// \remarks        n/a 
/// 
/// \param[out]     p_szPIN       Pointer to the already allocated buffer, which
///                               has to be used as destination buffer
/// \param[in]      p_nBuffSize   The size of the already defined destination 
///                               buffer
/// 
/// \return         BOOL          TRUE, if the PIN could be read; FALSE otherwise
/// 
/// \version        1.0 
/// 
//////////////////////////////////////////////////////////////////////////////// 
BOOL E2PROM_ReadPinValue( char* p_szPIN, uint8_t p_nBuffSize )
{
  BOOL bRetVal = FALSE;
  
  // 1. Test, if a PIN value is stored
  if (EEPReadByte(EEPROM_PIN_ADDRESS) == EEPROM_DEF)
  {
    // The first byte is still 0xFF (EEPROM_DEF); that means, that no PIN is stored
    bRetVal = FALSE;
  }
  else
  {
    bRetVal = ReadStringFromEEPROM(EEPROM_PIN_ADDRESS, p_szPIN, p_nBuffSize, TRUE);
  }
  
  return bRetVal;
}

//////////////////////////////////////////////////////////////////////////////// 
/// 
/// \brief          This function writes the given PIN to the EEPROM
/// 
/// \fn             BOOL E2PROM_WritePinValue(char* p_szPIN, uint8_t p_nBuffSize) 
/// 
/// \author         Michael
/// \date           30.12.2012 
/// 
/// \remarks        n/a 
/// 
/// \param[in]      p_szPIN 
/// \param[in]      p_nBuffSize 
/// 
/// \return         void 
/// 
/// \version        1.0 
/// 
//////////////////////////////////////////////////////////////////////////////// 
void E2PROM_WritePinValue( char* p_szPIN )
{
  WriteStringToEEPROM(EEPROM_PIN_ADDRESS, p_szPIN, TRUE);	
}
